/* zexy-setup autogenerated setup-file
 * generated by "./makesource.sh"
 * !! DO NOT MANUALLY EDIT  !!
 */

#include "z_zexy.h"

void z_zexy_setup(void)
{
	z_a2l_setup(); /* a2l.c */
	z_atoi_setup(); /* atoi.c */
	z_avg__setup(); /* avg~.c */
	z_blockmirror__setup(); /* blockmirror~.c */
	z_blockswap__setup(); /* blockswap~.c */
	z_date_setup(); /* date.c */
	z_demultiplex_setup(); /* demultiplex.c */
	z_demultiplex__setup(); /* demultiplex~.c */
	z_dfreq__setup(); /* dfreq~.c */
	z_dirac__setup(); /* dirac~.c */
	z_dot_setup(); /* dot.c */
	z_drip_setup(); /* drip.c */
	z_envrms__setup(); /* envrms~.c */
	z_fifop_setup(); /* fifop.c */
	z_glue_setup(); /* glue.c */
	z_index_setup(); /* index.c */
	z_length_setup(); /* length.c */
	z_lifop_setup(); /* lifop.c */
	z_limiter__setup(); /* limiter~.c */
	z_list2int_setup(); /* list2int.c */
	z_list2symbol_setup(); /* list2symbol.c */
	z_lister_setup(); /* lister.c */
	z_lpt_setup(); /* lpt.c */
	z_makesymbol_setup(); /* makesymbol.c */
	z_mavg_setup(); /* mavg.c */
	z_minmax_setup(); /* minmax.c */
	z_msgfile_setup(); /* msgfile.c */
	z_multiline__setup(); /* multiline~.c */
	z_multiplex_setup(); /* multiplex.c */
	z_multiplex__setup(); /* multiplex~.c */
	z_niagara_setup(); /* niagara.c */
	z_noish__setup(); /* noish~.c */
	z_noisi__setup(); /* noisi~.c */
	z_operating_system_setup(); /* operating_system.c */
	z_packel_setup(); /* packel.c */
	z_pack__setup(); /* pack~.c */
	z_pdf__setup(); /* pdf~.c */
	z_prime_setup(); /* prime.c */
	z_quantize__setup(); /* quantize~.c */
	z_repack_setup(); /* repack.c */
	z_repeat_setup(); /* repeat.c */
	z_sfplay_setup(); /* sfplay.c */
	z_sfrecord_setup(); /* sfrecord.c */
	z_sigzero__setup(); /* sigzero~.c */
	z_sort_setup(); /* sort.c */
	z_step__setup(); /* step~.c */
	z_strcmp_setup(); /* strcmp.c */
	z_sum_setup(); /* sum.c */
	z_swap__setup(); /* swap~.c */
	z_symbol2list_setup(); /* symbol2list.c */
	z_tabdump_setup(); /* tabdump.c */
	z_tabminmax_setup(); /* tabminmax.c */
	z_tabset_setup(); /* tabset.c */
	z_tavg__setup(); /* tavg~.c */
	z_time_setup(); /* time.c */
	z_unpack__setup(); /* unpack~.c */
	z_urn_setup(); /* urn.c */
	z_winNT_portio_setup(); /* winNT_portio.c */
	z_wrap_setup(); /* wrap.c */
	z_z_sigbin_setup(); /* z_sigbin.c */
	z_z__setup(); /* z~.c */
}

