/* zexy-setup autogenerated header-file
 * generated by "./makesource.sh"
 * !! DO NOT MANUALLY EDIT  !!
 */

#ifndef Z_ZEXY_H__
#define Z_ZEXY_H__
void z_a2l_setup(void); /* a2l.c */
void z_atoi_setup(void); /* atoi.c */
void z_avg__setup(void); /* avg~.c */
void z_blockmirror__setup(void); /* blockmirror~.c */
void z_blockswap__setup(void); /* blockswap~.c */
void z_date_setup(void); /* date.c */
void z_demultiplex_setup(void); /* demultiplex.c */
void z_demultiplex__setup(void); /* demultiplex~.c */
void z_dfreq__setup(void); /* dfreq~.c */
void z_dirac__setup(void); /* dirac~.c */
void z_dot_setup(void); /* dot.c */
void z_drip_setup(void); /* drip.c */
void z_envrms__setup(void); /* envrms~.c */
void z_fifop_setup(void); /* fifop.c */
void z_glue_setup(void); /* glue.c */
void z_index_setup(void); /* index.c */
void z_length_setup(void); /* length.c */
void z_lifop_setup(void); /* lifop.c */
void z_limiter__setup(void); /* limiter~.c */
void z_list2int_setup(void); /* list2int.c */
void z_list2symbol_setup(void); /* list2symbol.c */
void z_lister_setup(void); /* lister.c */
void z_lpt_setup(void); /* lpt.c */
void z_makesymbol_setup(void); /* makesymbol.c */
void z_mavg_setup(void); /* mavg.c */
void z_minmax_setup(void); /* minmax.c */
void z_msgfile_setup(void); /* msgfile.c */
void z_multiline__setup(void); /* multiline~.c */
void z_multiplex_setup(void); /* multiplex.c */
void z_multiplex__setup(void); /* multiplex~.c */
void z_niagara_setup(void); /* niagara.c */
void z_noish__setup(void); /* noish~.c */
void z_noisi__setup(void); /* noisi~.c */
void z_operating_system_setup(void); /* operating_system.c */
void z_packel_setup(void); /* packel.c */
void z_pack__setup(void); /* pack~.c */
void z_pdf__setup(void); /* pdf~.c */
void z_prime_setup(void); /* prime.c */
void z_quantize__setup(void); /* quantize~.c */
void z_repack_setup(void); /* repack.c */
void z_repeat_setup(void); /* repeat.c */
void z_sfplay_setup(void); /* sfplay.c */
void z_sfrecord_setup(void); /* sfrecord.c */
void z_sigzero__setup(void); /* sigzero~.c */
void z_sort_setup(void); /* sort.c */
void z_step__setup(void); /* step~.c */
void z_strcmp_setup(void); /* strcmp.c */
void z_sum_setup(void); /* sum.c */
void z_swap__setup(void); /* swap~.c */
void z_symbol2list_setup(void); /* symbol2list.c */
void z_tabdump_setup(void); /* tabdump.c */
void z_tabminmax_setup(void); /* tabminmax.c */
void z_tabset_setup(void); /* tabset.c */
void z_tavg__setup(void); /* tavg~.c */
void z_time_setup(void); /* time.c */
void z_unpack__setup(void); /* unpack~.c */
void z_urn_setup(void); /* urn.c */
void z_winNT_portio_setup(void); /* winNT_portio.c */
void z_wrap_setup(void); /* wrap.c */
void z_z_sigbin_setup(void); /* z_sigbin.c */
void z_z__setup(void); /* z~.c */
#endif /* Z_ZEXY_H__ */

